@@ -3,8 +3,7 @@ from terraform_compliance.common.helper import seek_key_in_dict, flatten_list, d
 import sys
 from copy import deepcopy
 from terraform_compliance.common.defaults import Defaults
-from terraform_compliance.common.exceptions import TerraformComplianceInternalFailure
-
+from terraform_compliance.extensions.cache import Cache
 
 class TerraformParser(object):
     def __init__(self, filename, parse_it=True):
@@ -35,8 +34,10 @@ class TerraformParser(object):
         self.configuration = dict(resources={}, variables={})
         self.file_type = "plan"
         self.resources_raw = {}
+        self.parse_it = parse_it
 
         if parse_it:
+            self.cache = Cache()
             self.parse()
 
     def _version_check(self):
@@ -84,6 +85,13 @@ class TerraformParser(object):
         :return: none
         '''
 
+        # Read Cache
+        if self.parse_it:
+            cache = self.cache.get('resources')
+            if cache:
+                self.resources = cache
+                return
+
         # Resources ( exists in Plan )
         for findings in seek_key_in_dict(self.raw.get('planned_values', {}).get('root_module', {}), 'resources'):
             for resource in findings.get('resources', []):
@@ -131,6 +139,9 @@ class TerraformParser(object):
                 else:
                     self.resources[resource['address']] = resource
 
+        if self.parse_it:
+            self.cache.set('resources', self.resources)
+
     def _parse_configurations(self):
         '''
         Assigns all configuration related data defined in the terraform plan. This is mostly used for
@@ -139,6 +150,13 @@ class TerraformParser(object):
         :return: none
         '''
 
+        # Read Cache
+        if self.parse_it:
+            cache = self.cache.get('configuration')
+            if cache:
+                self.configuration = cache
+                return
+
         # Resources
         self.configuration['resources'] = {}
 
@@ -200,6 +218,9 @@ class TerraformParser(object):
 
                 self.configuration['outputs'][key] = tmp_output
 
+        if self.parse_it:
+            self.cache.set('configuration', self.configuration)
+
     def _mount_resources(self, source, target, ref_type):
         '''
         Mounts values of the source resource to the target resource's values with ref_type key
@@ -218,7 +239,7 @@ class TerraformParser(object):
                     if target_resource not in self.resources or 'values' not in self.resources[target_resource]:
                         continue
 
-                    resource = deepcopy(self.resources[source_resource]['values'])
+                    resource = self.resources_raw[source_resource]['values']
                     resource[Defaults.mounted_ptr] = True
 
                     if Defaults.r_mount_ptr not in self.resources[target_resource]:
@@ -235,12 +256,16 @@ class TerraformParser(object):
                         self.resources[target_resource]['values'][ref_type].append(resource)
                         self.resources[target_resource][Defaults.r_mount_ptr][parameter] = ref_type
                         self.resources[target_resource][Defaults.r_mount_addr_ptr][parameter] = source
-                        self.resources[target_resource][Defaults.r_mount_addr_ptr_list].extend(source)
+                        target_set = set(self.resources[target_resource][Defaults.r_mount_addr_ptr_list])
+                        source_set = set(source)
+                        self.resources[target_resource][Defaults.r_mount_addr_ptr_list] = list(target_set | source_set)
                     else:
                         self.resources[target_resource]['values'][ref_type].append(resource)
                         self.resources[target_resource][Defaults.r_mount_ptr][parameter] = ref_type
                         self.resources[target_resource][Defaults.r_mount_addr_ptr][parameter] = source
-                        self.resources[target_resource][Defaults.r_mount_addr_ptr_list].extend(source)
+                        target_set = set(self.resources[target_resource][Defaults.r_mount_addr_ptr_list])
+                        source_set = set(source)
+                        self.resources[target_resource][Defaults.r_mount_addr_ptr_list] = list(target_set | source_set)
 
                     if parameter not in self.resources[source_resource]['values']:
                         self.resources[source_resource]['values'][parameter] = target_resource
@@ -352,7 +377,18 @@ class TerraformParser(object):
             self._parse_variables()
             self._parse_configurations()
 
-        self._mount_references()
+        cache = self.cache.get('mounted_resources') if self.parse_it else None
+
+        if cache:
+            # print('Read from cache, instead of re-mounting.')
+            self.resources = cache
+        else:
+            # print('Building cache for mounted resources at {}'.format(Defaults.cache_dir))
+            self._mount_references()
+
+            if self.parse_it:
+                self.cache.set('mounted_resources', self.resources)
+
         self._distribute_providers()
 
         for _, resource in self.resources.items():
